@@ -266,6 +266,14 @@ library if you like.
        "ytitle"      nil     # Title of y axis (if nil, internally determined)
        "xunits"      nil     # Units of x axis (if nil, internally determined)
        "yunits"      nil     # Units of y axis (if nil, internally determined)
+       "xtickint"    nil     # Interval of x axis tickmark
+                             #                 (if nil, internally determined)
+       "ytickint"    nil     # Interval of y axis tickmark
+                             #                 (if nil, internally determined)
+       "xlabelint"   nil     # Interval of x axis label
+                             #                 (if nil, internally determined)
+       "ylabelint"   nil     # Interval of y axis label
+                             #                 (if nil, internally determined)
        "help"        false   # show help message if true
 
     RETURN VALUE
@@ -795,13 +803,17 @@ module NumRu
     %w!gl sg sl sw uz ul uc uu us ud ue ug um!.each do |pkg|
       eval <<-EOS, nil, __FILE__, __LINE__+1
         def #{pkg}_set_params(hash)
-	  before = Hash.new
-	  hash.each{|k,v|
-	    before[k]=DCL.#{pkg}pget(k)
-	    DCL.#{pkg}pset(k,v)
-	  }
-	  before
-	end
+          before = Hash.new
+          hash.each{|k,v|
+            before[k]=DCL.#{pkg}pget(k)
+            if(v.is_a? String) then
+              DCL.#{pkg}cset(k,v)
+            else
+              DCL.#{pkg}pset(k,v)
+            end
+          }
+          before
+        end
       EOS
     end
 
@@ -817,16 +829,16 @@ module NumRu
       nlev = DCL.udqcln
       cont_params = Array.new
       for i in 1..nlev
-	cont_params.push( DCL.udqclv(i) )   # => [zlev,indx,ityp,clv,hl]
+        cont_params.push( DCL.udqclv(i) )   # => [zlev,indx,ityp,clv,hl]
       end
       DCL.udiclv     # clear the contours
 
       colors = clr_min + 
-	       NArray.int(nlev).indgen! * (clr_max-clr_min) / nlev
+               NArray.int(nlev).indgen! * (clr_max-clr_min) / nlev
 
       cont_params.sort!
       for i in 0...nlev
-	cont_params[i][1] += colors[i]*10   # indx += colors[i]*10
+        cont_params[i][1] += colors[i]*10   # indx += colors[i]*10
         DCL.udsclv(*cont_params[i])
       end
     end
@@ -850,26 +862,26 @@ module NumRu
       interval = options['interval']
       nozero = options['nozero']
       if interval
-	dx = interval
+        dx = interval
       elsif nlev
-	dx = -nlev
+        dx = -nlev
       else
-	dx = 0
+        dx = 0
       end
       if min || max
- 	min = v.min if !min
- 	max = v.max if !max
-	DCL.udgcla(min, max, dx)
+        min = v.min if !min
+        max = v.max if !max
+        DCL.udgcla(min, max, dx)
       else
-	DCL.udgclb(v, dx)
+        DCL.udgclb(v, dx)
       end
       if nozero
-	DCL.uddclv(0.0)
+        DCL.uddclv(0.0)
       end
       if options['coloring']
-	clr_min = ( options['clr_min'] || 13 )
-	clr_max = ( options['clr_max'] || 100 )
-	ud_coloring( clr_min, clr_max )
+        clr_min = ( options['clr_min'] || 13 )
+        clr_max = ( options['clr_max'] || 100 )
+        ud_coloring( clr_min, clr_max )
       end
     end
 
@@ -883,11 +895,11 @@ module NumRu
       # < check levels >
       case levels
       when Array, NArray
-	# This is expected. Nothing to do.
+        # This is expected. Nothing to do.
       when Numric
-	levels = [levels]
+        levels = [levels]
       else
-	raise ArgumentError, "invalid level specification (#{levels})"
+        raise ArgumentError, "invalid level specification (#{levels})"
       end
 
       nlev = levels.length
@@ -895,80 +907,80 @@ module NumRu
       # < index >
       case index
       when Array
-	raise ArgumentError, "index is an empty array" if index.length == 0
-	while (index.length < nlev )
-	  index += index
-	end
+        raise ArgumentError, "index is an empty array" if index.length == 0
+        while (index.length < nlev )
+          index += index
+        end
       when Numeric
-	index = [index]*nlev
+        index = [index]*nlev
       when nil
-	index = [DCL.udpget('indxmn')]*nlev
+        index = [DCL.udpget('indxmn')]*nlev
       else
-	raise ArgumentError, "unsupported index type (#{index.class})"
+        raise ArgumentError, "unsupported index type (#{index.class})"
       end
 
       # < line_type >
       case line_type
       when Array
-	raise ArgumentError, "line_type is an empty array" if line_type.length == 0
-	while (line_type.length < nlev )
-	  line_type += line_type
-	end
+        raise ArgumentError, "line_type is an empty array" if line_type.length == 0
+        while (line_type.length < nlev )
+          line_type += line_type
+        end
       when Numeric
-	line_type = [line_type]*nlev
+        line_type = [line_type]*nlev
       when nil
-	line_type = [1]*nlev
+        line_type = [1]*nlev
       else
-	raise ArgumentError, "unsupported index type (#{index.class})"
+        raise ArgumentError, "unsupported index type (#{index.class})"
       end
 
       # < label >
       case label
       when Array
-	raise ArgumentError, "label is an empty array" if label.length == 0
-	while (label.length < nlev )
-	  label += label
-	end
+        raise ArgumentError, "label is an empty array" if label.length == 0
+        while (label.length < nlev )
+          label += label
+        end
       when String
-	label = [label]*nlev
+        label = [label]*nlev
       when false
-	label = [""]*nlev
+        label = [""]*nlev
       when true
-	label = (0...nlev).collect{|i| 
-	    DCL.udlabl(levels[i])
-	}
+        label = (0...nlev).collect{|i| 
+            DCL.udlabl(levels[i])
+        }
       when nil
-	indxmj = DCL.udpget('indxmj')
-	label = (0...nlev).collect{|i| 
-	  if index[i]==indxmj
-	    DCL.udlabl(levels[i])
-	  else
-	    ""
-	  end
-	}
+        indxmj = DCL.udpget('indxmj')
+        label = (0...nlev).collect{|i| 
+          if index[i]==indxmj
+            DCL.udlabl(levels[i])
+          else
+            ""
+          end
+        }
       else
-	raise ArgumentError, "unsupported index type (#{index.class})"
+        raise ArgumentError, "unsupported index type (#{index.class})"
       end
 
       # < label_height >
       case label_height
       when Array
-	raise ArgumentError, "label_height is an empty array" if label_height.length == 0
-	while (label_height.length < nlev )
-	  label_height += label_height
-	end
+        raise ArgumentError, "label_height is an empty array" if label_height.length == 0
+        while (label_height.length < nlev )
+          label_height += label_height
+        end
       when Numeric
-	label_height = [label_height]*nlev
+        label_height = [label_height]*nlev
       when nil
-	label_height = label.collect{|lv| lv=="" ? 0.0 : DCL.udpget('rsizel')}
+        label_height = label.collect{|lv| lv=="" ? 0.0 : DCL.udpget('rsizel')}
       else
-	raise ArgumentError, "unsupported index type (#{index.class})"
+        raise ArgumentError, "unsupported index type (#{index.class})"
       end
 
       # < set levels >
 
       for i in 0...nlev
-	DCL.udsclv(levels[i],index[i],line_type[i],label[i],label_height[i])
+        DCL.udsclv(levels[i],index[i],line_type[i],label[i],label_height[i])
       end
       nil
     end
@@ -987,18 +999,18 @@ module NumRu
       nlev = options['nlev']
       interval = options['interval']
       if interval
-	dx = interval
+        dx = interval
       elsif nlev
-	dx = -nlev
+        dx = -nlev
       else
-	dx = 0
+        dx = 0
       end
       if min || max
- 	min = v.min if !min
- 	max = v.max if !max
-	DCL.uegtla(min, max, dx)
+        min = v.min if !min
+        max = v.max if !max
+        DCL.uegtla(min, max, dx)
       else
-	DCL.uegtlb(v, dx)
+        DCL.uegtlb(v, dx)
       end
     end
 
@@ -1012,10 +1024,10 @@ module NumRu
       # < check types >
 
       if !levels.is_a?(Array) && !levels.is_a?(NArray)
-	raise TypeError, "levels: Array or NArray expected (#{levels.inspect})"
+        raise TypeError, "levels: Array or NArray expected (#{levels.inspect})"
       end
       if !patterns.is_a?(Array) && !patterns.is_a?(NArray)
-	raise TypeError, "patterns: Array or NArray expected (#{patterns.inspect})"
+        raise TypeError, "patterns: Array or NArray expected (#{patterns.inspect})"
       end
 
       # < set levels >
@@ -1025,23 +1037,23 @@ module NumRu
 
       case (nlev - npat)
       when 1
-	for i in 0...nlev-1
-	  DCL.uestlv(levels[i],levels[i+1],patterns[i])
-	end
+        for i in 0...nlev-1
+          DCL.uestlv(levels[i],levels[i+1],patterns[i])
+        end
       when 0
-	for i in 0...nlev-1
-	  DCL.uestlv(levels[i],levels[i+1],patterns[i])
-	end
-	DCL.uestlv(levels[-1],DCL.glpget('rmiss'),patterns[-1])
+        for i in 0...nlev-1
+          DCL.uestlv(levels[i],levels[i+1],patterns[i])
+        end
+        DCL.uestlv(levels[-1],DCL.glpget('rmiss'),patterns[-1])
       when -1
-	DCL.uestlv(DCL.glpget('rmiss'),levels[0],patterns[0])
-	for i in 1...nlev
-	  DCL.uestlv(levels[i-1],levels[i],patterns[i])
-	end
-	DCL.uestlv(levels[-1],DCL.glpget('rmiss'),patterns[-1])
+        DCL.uestlv(DCL.glpget('rmiss'),levels[0],patterns[0])
+        for i in 1...nlev
+          DCL.uestlv(levels[i-1],levels[i],patterns[i])
+        end
+        DCL.uestlv(levels[-1],DCL.glpget('rmiss'),patterns[-1])
       else
-	raise ArgumentError, 
-	  "lengths of levels(#{nlev}) and patterns(#{npat}) are inconsistent"
+        raise ArgumentError, 
+          "lengths of levels(#{nlev}) and patterns(#{npat}) are inconsistent"
       end
       nil
     end
@@ -1081,8 +1093,8 @@ module NumRu
       vx = vxmax + dvx
       vy = vymax - charsize/2
       str_ary.each{|str|
-	DCL::sgtxzv(vx,vy,str,charsize,0,-1,1)
-	vy -= dvy
+        DCL::sgtxzv(vx,vy,str,charsize,0,-1,1)
+        vy -= dvy
       }
       nil
     end
@@ -1102,54 +1114,54 @@ module NumRu
     @@next_fig = nil
     def next_fig(options)
       if options.is_a?(Hash)
-	@@next_fig = options
+        @@next_fig = options
       else
-	raise TypeError,"Hash expected"
+        raise TypeError,"Hash expected"
       end
       nil
     end
 
     def fig(xax, yax, options=nil)
       if @@next_fig
-	options = ( options ? @@next_fig.update(options) : @@next_fig )
-	@@next_fig = nil
+        options = ( options ? @@next_fig.update(options) : @@next_fig )
+        @@next_fig = nil
       end
       opts = @@fig.interpret(options)
       if (xreverse=opts['xreverse']).is_a?(String)
-	atts = opts['xreverse'].split(',').collect{|v| v.split(':')}
-	xreverse = false
-	atts.each{|key,val| 
-	  xreverse = ( xax.get_att(key) == val )
-	  break if xreverse
-	}
+        atts = opts['xreverse'].split(',').collect{|v| v.split(':')}
+        xreverse = false
+        atts.each{|key,val| 
+          xreverse = ( xax.get_att(key) == val )
+          break if xreverse
+        }
       end
       if (yreverse=opts['yreverse']).is_a?(String)
-	atts = opts['yreverse'].split(',').collect{|v| v.split(':')}
-	yreverse = false
-	atts.each{|key,val| 
-	  yreverse = ( yax.get_att(key) == val )
-	  break if yreverse
-	}
+        atts = opts['yreverse'].split(',').collect{|v| v.split(':')}
+        yreverse = false
+        atts.each{|key,val| 
+          yreverse = ( yax.get_att(key) == val )
+          break if yreverse
+        }
       end
       if !opts['window']
-	if xreverse
-	  xrange = [ xax.max, xax.min ]
-	else
-	  xrange = [ xax.min, xax.max ]
-	end
-	if yreverse
-	  yrange = [ yax.max, yax.min ]
-	else
-	  yrange = [ yax.min, yax.max ]
-	end
-	window=[xrange[0], xrange[1], yrange[0], yrange[1]]
+        if xreverse
+          xrange = [ xax.max, xax.min ]
+        else
+          xrange = [ xax.min, xax.max ]
+        end
+        if yreverse
+          yrange = [ yax.max, yax.min ]
+        else
+          yrange = [ yax.min, yax.max ]
+        end
+        window=[xrange[0], xrange[1], yrange[0], yrange[1]]
       else
-	window=opts['window']
+        window=opts['window']
       end
       if opts['new_frame']
-	DCL.grfrm
+        DCL.grfrm
       else
-	DCL.grfig
+        DCL.grfig
       end
       DCL.grsvpt(*opts['viewport'])
       DCL.grswnd(*window)
@@ -1164,7 +1176,15 @@ module NumRu
       ['xtitle',  nil,  'Title of x axis (if nil, internally determined)'],
       ['ytitle',  nil,  'Title of y axis (if nil, internally determined)'],
       ['xunits',  nil,  'Units of x axis (if nil, internally determined)'],
-      ['yunits',  nil,  'Units of y axis (if nil, internally determined)']
+      ['yunits',  nil,  'Units of y axis (if nil, internally determined)'],
+      ['xtickint',  nil,
+            'Interval of x axis tickmark (if nil, internally determined)'],
+      ['ytickint',  nil,
+            'Interval of y axis tickmark (if nil, internally determined)'],
+      ['xlabelint',  nil,
+               'Interval of x axis label (if nil, internally determined)'],
+      ['ylabelint',  nil,
+               'Interval of y axis label (if nil, internally determined)']
     )
     def set_axes(options)
       @@axes.set(options)
@@ -1173,58 +1193,62 @@ module NumRu
     @@next_axes = nil
     def next_axes(options)
       if options.is_a?(Hash)
-	@@next_axes = options
+        @@next_axes = options
       else
-	raise TypeError,"Hash expected"
+        raise TypeError,"Hash expected"
       end
       nil
     end
 
     def axes(xax=nil, yax=nil, options=nil)
       if @@next_axes
-	options = ( options ? @@next_axes.update(options) : @@next_axes )
-	@@next_axes = nil
+        options = ( options ? @@next_axes.update(options) : @@next_axes )
+        @@next_axes = nil
       end
       opts = @@axes.interpret(options)
       if opts['xside'].length > 0
-	xai = _get_axinfo(xax)
-	DCL.uscset('cxttl', (opts['xtitle'] || xai['title']) )
-	DCL.uscset('cxunit',(opts['xunits'] || xai['units']) )
-	opts['xside'].split('').each{|s| DCL.usxaxs(s)}  # scan('.') also works
+        xai = _get_axinfo(xax)
+        DCL.uscset('cxttl', (opts['xtitle'] || xai['title']) )
+        DCL.uscset('cxunit',(opts['xunits'] || xai['units']) )
+        DCL.uspset('dxt', opts['xtickint'])  if(opts['xtickint'])
+        DCL.uspset('dxl', opts['xlabelint']) if(opts['xlabelint'])
+        opts['xside'].split('').each{|s| DCL.usxaxs(s)}  # scan('.') also works
       end
       if opts['yside'].length > 0
-	yai = _get_axinfo(yax)
-	DCL.uscset('cyttl', (opts['ytitle'] || yai['title']) )
-	DCL.uscset('cyunit',(opts['yunits'] || yai['units']) )
-	opts['yside'].split('').each{|s| DCL.usyaxs(s)}  # scan('.') also works
+        yai = _get_axinfo(yax)
+        DCL.uscset('cyttl', (opts['ytitle'] || yai['title']) )
+        DCL.uscset('cyunit',(opts['yunits'] || yai['units']) )
+        DCL.uspset('dxt', opts['ytickint'])  if(opts['ytickint'])
+        DCL.uspset('dxl', opts['ylabelint']) if(opts['ylabelint'])
+        opts['yside'].split('').each{|s| DCL.usyaxs(s)}  # scan('.') also works
       end
       nil
     end
 
     def _get_axinfo(vary)
       if vary.is_a?(VArray)
-	raise "Not 1D" if vary.rank!=1
-	hash = {
-	        'title'=>(vary.get_att('long_name') || vary.name), #.gsub('_','' )
-	        'units'=>(vary.get_att('units') || '')             #.gsub('_',' ')
+        raise "Not 1D" if vary.rank!=1
+        hash = {
+                'title'=>(vary.get_att('long_name') || vary.name), #.gsub('_','' )
+                'units'=>(vary.get_att('units') || '')             #.gsub('_',' ')
                }
       elsif vary.nil?
-	hash = {
-	        'title'=>'',
-	        'units'=>''
+        hash = {
+                'title'=>'',
+                'units'=>''
                }
       else
-	raise TypeError,"not a VArray or nil" 
+        raise TypeError,"not a VArray or nil" 
       end
       hash 
     end
 
     def line(gphys, newframe=true, options=nil)
       if newframe!=true && newframe!=false
-	raise ArgumentError, "2nd arg (newframe) must be true or false"
+        raise ArgumentError, "2nd arg (newframe) must be true or false"
       end
       if ! defined?(@@line_options)
-	@@line_options = Misc::KeywordOptAutoHelp.new(
+        @@line_options = Misc::KeywordOptAutoHelp.new(
           ['title', nil, 'Title of the figure(if nil, internally determined)'],
           ['annotate', true, 'if false, do not put texts on the right margin even when newframe==true'],
           ['exchange', false, 'whether to exchange x and y axes'],
@@ -1235,17 +1259,17 @@ module NumRu
       opts = @@line_options.interpret(options)
       gp = gphys.first1D
       if !opts['exchange']
-	x = gp.coord(0)
-	y = gp.data
+        x = gp.coord(0)
+        y = gp.data
       else
-	y = gp.coord(0)
-	x = gp.data
+        y = gp.coord(0)
+        x = gp.data
       end
       if newframe
-	fig(x, y)
-	axes(x, y)
-	title( (opts['title'] || gp.data.get_att('long_name')) )
-	annotate(gp.lost_axes) if opts['annotate']
+        fig(x, y)
+        axes(x, y)
+        title( (opts['title'] || gp.data.get_att('long_name')) )
+        annotate(gp.lost_axes) if opts['annotate']
       end
       DCL.uulinz(x.val, y.val, opts['type'], opts['index'])
       nil
@@ -1253,10 +1277,10 @@ module NumRu
 
     def mark(gphys, newframe=true, options=nil)
       if newframe!=true && newframe!=false
-	raise ArgumentError, "2nd arg (newframe) must be true or false"
+        raise ArgumentError, "2nd arg (newframe) must be true or false"
       end
       if ! defined?(@@mark_options)
-	@@mark_options = Misc::KeywordOptAutoHelp.new(
+        @@mark_options = Misc::KeywordOptAutoHelp.new(
           ['title', nil, 'Title of the figure(if nil, internally determined)'],
           ['annotate', true, 'if false, do not put texts on the right margin even when newframe==true'],
           ['exchange', false, 'whether to exchange x and y axes'],
@@ -1268,17 +1292,17 @@ module NumRu
       opts = @@mark_options.interpret(options)
       gp = gphys.first1D
       if !opts['exchange']
-	x = gp.coord(0)
-	y = gp.data
+        x = gp.coord(0)
+        y = gp.data
       else
-	y = gp.coord(0)
-	x = gp.data
+        y = gp.coord(0)
+        x = gp.data
       end
       if newframe
-	fig(x, y, opts['fig_params'])
-	axes(x, y)
-	title( (opts['title'] || gp.data.get('long_name')) )
-	annotate(gp.lost_axes) if opts['annotate']
+        fig(x, y, opts['fig_params'])
+        axes(x, y)
+        title( (opts['title'] || gp.data.get('long_name')) )
+        annotate(gp.lost_axes) if opts['annotate']
       end
       DCL.uumrkz(x.val, y.val, opts['type'], opts['index'], opts['size'])
       nil
@@ -1300,9 +1324,9 @@ module NumRu
     @@next_linear_contour_options = nil
     def next_linear_contour_options(options)
       if options.is_a?(Hash)
-	@@next_linear_contour_options = options
+        @@next_linear_contour_options = options
       else
-	raise TypeError,"Hash expected"
+        raise TypeError,"Hash expected"
       end
       nil
     end
@@ -1322,7 +1346,7 @@ module NumRu
     def _set_contour_levels_(opts)
       raise ArgumentError, "'levels' must be explicitly set" if !opts['levels']
       DCLExt.ud_set_contour(opts['levels'],opts['index'],opts['line_type'],
-		  opts['label'],opts['label_height'])
+                  opts['label'],opts['label_height'])
       @@set_contour_levels=true
       nil
     end
@@ -1335,10 +1359,10 @@ module NumRu
 
     def contour(gphys, newframe=true, options=nil)
       if newframe!=true && newframe!=false
-	raise ArgumentError, "2nd arg (newframe) must be true or false"
+        raise ArgumentError, "2nd arg (newframe) must be true or false"
       end
       if ! defined?(@@contour_options)
-	@@contour_options = Misc::KeywordOptAutoHelp.new(
+        @@contour_options = Misc::KeywordOptAutoHelp.new(
           ['title', nil, 'Title of the figure(if nil, internally determined)'],
           ['annotate', true, 'if false, do not put texts on the right margin even when newframe==true'],
           @@linear_contour_options,
@@ -1346,24 +1370,24 @@ module NumRu
         )
       end
       if @@next_linear_contour_options
-	options = ( options ? @@next_linear_contour_options.update(options) : 
-		              @@next_linear_contour_options )
-	@@next_linear_contour_options = nil
+        options = ( options ? @@next_linear_contour_options.update(options) : 
+                              @@next_linear_contour_options )
+        @@next_linear_contour_options = nil
       end
       opts = @@contour_options.interpret(options)
       gp = gphys.first2D
       xax = gp.coord(0)
       yax = gp.coord(1)
       if newframe
-	fig(xax, yax)
-	axes(xax, yax)
-	title( (opts['title'] || gp.data.get_att('long_name')) )
-	annotate(gp.lost_axes) if opts['annotate']
+        fig(xax, yax)
+        axes(xax, yax)
+        title( (opts['title'] || gp.data.get_att('long_name')) )
+        annotate(gp.lost_axes) if opts['annotate']
       end
       if opts['levels']
-	backup = @@set_contour_levels
-	_set_contour_levels_(opts)
-	@@set_contour_levels = backup
+        backup = @@set_contour_levels
+        _set_contour_levels_(opts)
+        @@set_contour_levels = backup
       elsif !@@set_contour_levels
         DCLExt.ud_set_linear_levs(gp.data.val, opts)
       end
@@ -1385,9 +1409,9 @@ module NumRu
     @@next_linear_tone_options = nil
     def next_linear_tone_options(options)
       if options.is_a?(Hash)
-	@@next_linear_tone_options = options
+        @@next_linear_tone_options = options
       else
-	raise TypeError,"Hash expected"
+        raise TypeError,"Hash expected"
       end
       nil
     end
@@ -1415,16 +1439,16 @@ module NumRu
 
     def tone(gphys, newframe=true, options=nil)
       if newframe!=true && newframe!=false
-	raise ArgumentError, "2nd arg (newframe) must be true or false"
+        raise ArgumentError, "2nd arg (newframe) must be true or false"
       end
       if ! defined?(@@tone_options)
-	@@tone_options = Misc::KeywordOptAutoHelp.new(
+        @@tone_options = Misc::KeywordOptAutoHelp.new(
           ['title', nil, 'Title of the figure(if nil, internally determined)'],
           ['annotate', true, 'if false, do not put texts on the right margin even when newframe==true'],
           ['ltone', true, 'Same as udpack parameter ltone'],
           @@linear_tone_options,
           @@tone_levels
-	)
+        )
       end
       opts = @@tone_options.interpret(options)
 
@@ -1432,26 +1456,26 @@ module NumRu
       xax = gp.coord(0)
       yax = gp.coord(1)
       if newframe
-	fig(xax, yax)
+        fig(xax, yax)
       end
       DCL.uwsgxa(xax.val)
       DCL.uwsgya(yax.val)
       if opts['levels'] && opts['patterns']
-	backup = @@set_tone_levels
-	_set_tone_levels_(opts)
-	@@set_tone_levels = backup
-	DCL.uetone(gp.data.val)
+        backup = @@set_tone_levels
+        _set_tone_levels_(opts)
+        @@set_tone_levels = backup
+        DCL.uetone(gp.data.val)
       elsif !@@set_tone_levels && opts['ltone']
         DCLExt.ue_set_linear_levs(gp.data.val, opts)
-	DCL.uetone(gp.data.val)
-	DCL.ueitlv
+        DCL.uetone(gp.data.val)
+        DCL.ueitlv
       else
-	DCL.uetone(gp.data.val)
+        DCL.uetone(gp.data.val)
       end
       if newframe
-	axes(xax, yax)  # axes should be drawn after uetone
-	title(gp.data.get_att('long_name'))
-	annotate(gp.lost_axes) if opts['annotate']
+        axes(xax, yax)  # axes should be drawn after uetone
+        title( (opts['title'] || gp.data.get_att('long_name')) )
+        annotate(gp.lost_axes) if opts['annotate']
       end
       nil
     end
@@ -1521,9 +1545,9 @@ if $0 == __FILE__
   GGraph.next_fig('itr'=>2)
   GGraph.next_axes('yunits'=>'','xunits'=>'')
   GGraph.line(gphys.cut(true,35,true).average(0), true, 
-	      'exchange'=>true, 'index'=>3, 'title'=>'TEMERATURE', 'annotate'=>false)
+              'exchange'=>true, 'index'=>3, 'title'=>'TEMERATURE', 'annotate'=>false)
   GGraph.mark(gphys.cut(true,35,true).average(0), false, 
-	      'exchange'=>true, 'type'=>3)
+              'exchange'=>true, 'type'=>3)
   #/ graph 3 /
   GGraph.contour(gphys)
   #/ graph 4 /
@@ -1542,7 +1566,7 @@ if $0 == __FILE__
   DCL.udpset('lmsg',true)
   #/ graph 8 /
   GGraph.set_contour_levels('levels'=>[-10,-5,0,5,10],'index'=>[3,1],
-			'line_type'=>2)
+                        'line_type'=>2)
   GGraph.contour(gphys)
   GGraph.clear_contour_levels
   #/ graph 9 /
