@@ -20,6 +20,11 @@ module NumRu
       attr_reader :grid, :data
       protected :grid
 
+      def grid_copy
+	# deep clone of the grid
+	@grid.copy
+      end
+
       def copy
 	 # deep clone onto memory
 	 GPhys.new( @grid.copy, @data.copy )
@@ -35,6 +40,10 @@ module NumRu
       def name=(nm)
 	 data.name=nm
       end
+      def rename(nm)
+	data.name=nm
+	self
+      end
 
       def val
 	 @data.val
@@ -42,20 +51,48 @@ module NumRu
       def val=(v)
 	 @data.val= v
       end
+      def replace_val(v)
+	 @data.replace_val(v)
+      end
+      def units
+	@data.units
+      end
+      def units=(units)
+	@data.units= units
+      end
 
       def [](*slicer)
-         slicer = __rubber_expansion( slicer )
+	 if slicer.length==1 && slicer[0].is_a?(Hash)
+	   slicer = __process_hash_slicer(slicer[0])
+	 else
+	   slicer = __rubber_expansion( slicer )
+	 end
 	 GPhys.new( @grid[*slicer], @data[*slicer] )
       end
 
       def []=(*args)
-         args = __rubber_expansion( args )
 	 val = args.pop
-	 val = val.data if val.is_a?(GPhys)
 	 slicer = args
+	 if slicer.length==1 && slicer[0].is_a?(Hash)
+	   slicer = __process_hash_slicer(slicer[0])
+	 else
+	   slicer = __rubber_expansion( slicer )
+	 end
+	 val = val.data if val.is_a?(GPhys)
 	 @data[*slicer] = val
       end
 
+      def __process_hash_slicer(hash)
+	raise ArgumentError, "Expect a Hash" if !hash.is_a?(Hash)
+	if (hash.keys - axnames).length > 0
+	  raise ArgumentError,"One or more of the hash keys "+
+	    "(#{hash.keys.inspect}) are not found in the axis names "+
+            "(#{axnames.inspect})."
+	end
+	axnames.collect{|nm| hash[nm] || true}   # slicer for []/[]=
+      end
+      private :__process_hash_slicer
+
       def cut( *args )
 	newgrid, slicer = @grid.cut( *args )
 	GPhys.new( newgrid, @data[ *slicer ] )
@@ -82,14 +119,37 @@ module NumRu
 	@grid.rank
       end
       def axis(i)
-	@grid.get_axis(i)
+	@grid.axis(i)
       end
       def coord(i)
-	@grid.get_axis(i).pos
+	@grid.axis(i).pos
       end
       def lost_axes
 	 @grid.lost_axes
       end
+      def dim_index( dimname )
+	 @grid.dim_index( dimname )
+      end
+
+      def coerce(other)
+	case other
+	when Numeric
+	  na_other = self.data.val.fill(other)   # Not efficient!
+           #developpers memo: perhaps need to let VArray know GPhys to avoid it
+           #XJ[rank[ GPhys `邪ƓB
+           #NArrayScalarł shape  [1] ԂĂ܂B
+	  va_other, = self.data.coerce(na_other)
+	  c_other = GPhys.new( @grid, va_other )
+	when Array, NArray
+	  va_other, = self.data.coerce(other)
+	  c_other = GPhys.new( @grid, va_other )
+	when VArray
+	  c_other = GPhys.new( @grid, other )
+	else
+	  raise "Cannot coerse #{other.class}"
+	end
+	[c_other, self]
+      end
 
       def shape_coerce(other)
 	 #
@@ -127,8 +187,8 @@ module NumRu
          #   raise ArgumentError, "Not a GPhys" if !gphys.is_a?(GPhys)
 	 #   GPhys.new( gphys.grid, VArray.#{f}(gphys.data) )
 	 #end
-	 def #{f}
-	    GPhys.new( self.grid, self.data.#{f} )
+	 def #{f}(*arg)
+	    GPhys.new( self.grid, self.data.#{f}(*arg) )
 	 end
          EOS
       end
@@ -143,18 +203,18 @@ module NumRu
 		 else
 		   vary = myself.data#{f} other
 		 end
-		 GPhys.new( myself.grid, vary )
+		 GPhys.new( myself.grid.copy, vary )
 	       else
 		 if other.is_a?(GPhys)
 		   vary = myself#{f} other.data
 		 else
 		   vary = myself#{f} other
 		 end
-		 GPhys.new( other.grid, vary )
+		 GPhys.new( other.grid.copy, vary )
 	       end
 	    else
 	       vary = self.data#{f} other
-	       GPhys.new( @grid, vary )
+	       GPhys.new( @grid.copy, vary )
 	    end
 	 end
 	 EOS
@@ -163,7 +223,7 @@ module NumRu
 	 eval <<-EOS, nil, __FILE__, __LINE__+1
 	 def #{f.delete(".")}(other)
             # returns NArray
-            self.data#{f} (other.is_a?(GPhys) ? other.data : other)
+            self.data#{f}(other.is_a?(GPhys) ? other.data : other)
 	 end
 	 EOS
       end
@@ -171,14 +231,14 @@ module NumRu
 	 eval <<-EOS, nil, __FILE__, __LINE__+1
 	 def #{f}
             vary = #{f.delete("@")} self.data
-	    GPhys.new( @grid, vary )
+	    GPhys.new( @grid.copy, vary )
 	 end
 	 EOS
       end
       for f in VArray::NArray_type1_methods
 	 eval <<-EOS, nil, __FILE__, __LINE__+1
 	 def #{f}(*args)
-	    GPhys.new( self.grid, self.data.#{f}(*args) )
+	    GPhys.new( self.grid.copy, self.data.#{f}(*args) )
 	 end
 	 EOS
       end
@@ -194,7 +254,7 @@ module NumRu
 	 def #{f}(*args)
             args = args.collect{|i| @grid.dim_index(i)}
 	    result = self.data.#{f}(*args)
-            if result.is_a?(Numeric)
+            if Numeric===result || UNumeric===result
 	      result
 	    else
 	      GPhys.new( self.grid.delete_axes(args, "#{f}"), result )
@@ -310,12 +370,18 @@ if $0 == __FILE__
 
    z = VArray.new( NArray.float(vx.length, vy.length).indgen! )
    p z.val
-   w = VArray.new( NArray.float(vx.length, vy.length).random! )
+   w = VArray.new( NArray.float(vx.length, vy.length).indgen!/10 ) # .random!
 
    gpz = GPhys.new(grid,z)
    gg = gpz[true,[0,2,1]]
    p '###',gg.val
    p gg[true,1].val
+   p gg['y'=>1].val
+   gg['y'=>1] = 999
+   p gg.val
+   p gg.coord(0).val, gg.coord(1).val
+   p gg.cut([1.2,3.8],1.1).val
+
    gpw = GPhys.new(grid,w)
    p '@@@',gpw.average(1).val
    p ( (gpz + gpw).val )
