@@ -1,5 +1,4 @@
 require "numru/gphys/gphys"
-require "numru/netcdf_miss"
 
 =begin
 =module NumRu::GPhys::NetCDF_Convention_Users_Guide
@@ -90,6 +89,159 @@ a NetCDF read/write helper by automatically interpreting conventions
     RETURN VALUE
     * nil
 
+---write_grid(file, grid_or_gphys)
+
+    Same as ((<write>)) but for writing only the contents of the grid.
+    (Used in ((<write>)).)
+
+    ARGUMENTS
+    * file (NetCDF): the NetCDF file to write in. Must be writable of course.
+    * grid_or_gphys (Grid or GPhys):
+
+    RETURN VALUE
+    * nil
+
+---each_along_dims_write(gphyses, files, *loopdims){...}  # a block is expected
+
+    Iterator to process GPhys objects too big to read on memory at once.
+    Makes a loop (loops) by dividing the GPhys object(s) (((|gphyses|)))
+    with the dimension(s) specified by ((|loopdims|)), and the results
+    (which is the return value of the block) are written in ((|files|)).
+
+    ARGUMENTS
+    * gphyses (GPhys or Array of GPhys): GPhys object(s) to be processed
+    * files (NetCDF or Array of NetCDF): the file in which the results are
+      written. The number of the file must be smaller than or equalt to 
+      the number of resultant GPhys objects (following the multiple assignment
+      rule of Ruby).
+    * loopdims (Array of String or Integer) : name (when String) or
+      count starting from zero (when Integer) 
+    * expected block : Number of arguments == number of GPhys objects in
+      ((|gphyses|)). Expected return value is an Array of GPhys objects
+      to be written ((|files|)).
+
+    RETURN VALUE
+    * GPhys objects in which the results are written
+
+    ERRORS
+
+    The following raise exceptions (in adition to errors in arguments).
+
+    * Shape of the GPhys objects in ((|gphyses|)) are not the same.
+    * Return value of the block is not an Array of GPhys.
+    * Dimension(s) used for looping (((|loopdims|))) is(are) eliminated
+      from the retunred GPhys objects.
+
+    USAGE
+
+    * EXAMPLE 1
+
+      Suppose that you want to do the following:
+
+        in = GPhys::NetCDF_IO.open(infile, varname)
+        ofile = NetCDF.create(ofilename)
+        out = in.mean(0)
+        GPhys::NetCDF_IO.write( ofile, out )
+        ofile.close
+
+      The data object (((|in|))) is read on memory and an averagin is made.
+      If the size of the data is too big to read on memory at once, you can
+      divid this process by using this iterator. The following gives the 
+      same result as above, but the processing is made for each subset
+      divided at the last dimension (represented by -1, as in the negative
+      indexing of Array).
+
+        in = GPhys::NetCDF_IO.open(infile, varname)
+        ofile = NetCDF.create(ofilename)
+        out = GPhys::NetCDF_IO.each_along_dims_write(in, ofile, -1){|in_sub|
+          [ in_sub.mean(0) ]
+        }
+        ofile.close
+
+      In this case, each_along_dims_write makes a loop by substituting
+      ((|in[false,0..0]|)), ((|in[false,1..1]|)), ((|in[false,2..2]|)),.. 
+      into the argument of the block (((|in_sub|))). Thus, the return
+      value of the block (here, ((|[ in_sub.mean(0) ]|))) consists of
+      ((|in[false,0..0].mean(0)|)), ((|in[false,1..1].mean(0)|)),.. .
+      This iterator creates a GPhys object in ((|out|)) that 
+      represents the whole part of the results (here, ((|in.mean(0)|))), and 
+      write the resultant subsets in it one by one. Therefore, the output file
+      is filled correctly when exiting the iterator.
+
+      Note that the subset (((|in_sub|))) retains the last dimension
+      but the length is 1 becasue of the slicing by Range (0..0, 1..1,..).
+      Therefore, the subset has the same rank as the original.
+      The output GPhys objects, as given by the return value of the block,
+      must have the dimension retained, since the dimension (whose length 
+      is one) is replaced by the original one when written in the file.
+      Therefore, THE FOLLOWING CAUSE AN ERROR (an exception is raised):
+
+        out = GPhys::NetCDF_IO.each_along_dims_write(in, ofile, 0){|in_sub|
+          [ in_sub.mean(0) ]
+        }
+
+      Here, looping is made by the first dimension (0), but it is eliminated 
+      from the result by averaging with the same dimension. (Also, note
+      that this averaging is non-sense, since the length of the first
+      dimension of the subset is 1).
+
+    * EXAMPLE 2
+
+      You can specify mutiple dimensions for looping to further
+      decrease the size of data to read on memory:
+
+        GPhys::NetCDF_IO.each_along_dims_write(in, ofile, -2, -1){|in_sub|
+          ...
+        }
+
+      Also, you can specify the loop dimension(s) by name(s):
+
+        GPhys::NetCDF_IO.each_along_dims_write(in, ofile, "y"){|in_sub|
+          ...
+        }
+
+        GPhys::NetCDF_IO.each_along_dims_write(in, ofile, "y", "z"){|in_sub|
+          ...
+        }
+
+    * EXAMPLE 3
+
+      You can give multiple objects in the iterotor if they
+      have the same shape (in future, this restriction may been loosened),
+      as follows:
+
+        in1 = GPhys::NetCDF_IO.open(infile1, varname1)
+        in2 = GPhys::NetCDF_IO.open(infile2, varname2)
+        in3 = GPhys::NetCDF_IO.open(infile3, varname3)
+        ofile = NetCDF.create(ofilename)
+        outA, outB = \
+          GPhys::NetCDF_IO.each_along_dims_write([in1,in2,in3], ofile, -1){
+            |isub1,isub2,isub3|
+            osubA = (isub1*isub2).mean(0)
+            osubB = (isub2*isub3).mean(1)
+            [ osubA, osubB ]
+          }
+        ofile.close
+
+      In this case, two output objects (outA and outB) are made 
+      from the three input objects (in1,in2,in3) and written in a
+      single file (ofile). If you want to separate into two files, 
+      you can do it like this: 
+
+        in1 = GPhys::NetCDF_IO.open(infile1, varname1)
+        in2 = GPhys::NetCDF_IO.open(infile2, varname2)
+        in3 = GPhys::NetCDF_IO.open(infile3, varname3)
+        ofile1 = NetCDF.create(ofilename1)
+        ofile2 = NetCDF.create(ofilename2)
+        outA, outB = \
+          GPhys::NetCDF_IO.each_along_dims_write([in1,in2,in3], [ofile1,ofile2], -1){
+            |isub1,isub2,isub3|
+            osubA = (isub1*isub2).mean(0)
+            osubB = (isub2*isub3).mean(1)
+            [ osubA, osubB ]
+          }
+        ofile.close
+
 ---set_convention(convention)
     Set a NetCDF convention to be interpreted.
 
@@ -111,57 +263,110 @@ module NumRu
 
   class GPhys
 
-    module NetCDF_Convention_Users_Guide
-      # NetCDF Convention of the NetCDF User's Guide
-      # To be used by (not to be included in) NetCDF_IO.
+    module IO_Common
+      # Functions independent of specific file formart.
+      # In future, this module will be separated in an independent
+      # file. (At this moment, it is placed here, since it is 
+      # small and used only by NetCDF_IO).
 
       module_function
 
-      def interpret_miss_handling_and_scaling(ncvar)
-	# CAUTION: WON'T WORK! -- For consistency, the behavior 
-        # of all NetCDFVar under VArrayNetCDF should be consolidated.
-	def ncvar.get(*args)
-	  get_with_miss_and_scaling(*args)
+      def each_along_dims_write(gphyses, files, loopdims, io_module)
+	case gphyses
+	when Array
+	  shape = gphyses[0].shape
+	  for i in 1...gphyses.length
+	    if shape != gphyses[i].shape
+	      raise "Shape of the GPhys objects must be the same"
+	    end
+	  end
+	else
+	  gphyses = [gphyses]     # put in an Array (if a single GPhys)
 	end
-	def ncvar.current_put(*args)
-	  put_with_miss_and_scaling(*args)
+	gp = gphyses[0]
+
+	if !files.is_a?(Array)
+	  files = [files]     # put in an Array (if a single File)
 	end
-	nil
-      end
 
-      module Miss_Handling_And_Saling
-	def get(*args)
-	  get_with_miss_and_scaling(*args)
+	if !loopdims.is_a?(Array)
+	  loopdims = [loopdims]  # put in an Array (if a single Integer/String)
 	end
-	def current_put(*args)
-	  put_with_miss_and_scaling(*args)
+	if loopdims.length == 0
+	  raise ArgumentError, "No loop dimension is specified "+
+	    " -- In that case, you don't need this iterator."
 	end
-      end
 
-      def coord_var_names(ncvar)
-	# name of the coordinate variables (to be the "pos" object in Axis)
-	ncvar.dim_names
-      end
+	#if loopdims.min<0 || loopdims.max>=gp.rank
+	#  raise ArguemntError,"Invalid dims #{loopdims.inspect} for #{gp.rank}D array"
+	#end
 
-      def cell_bounds?(coord_var)
-	# whether the coordinate variale represent grid cell bounds.
-	# coordvar (VArray)
-	# return value:
-	result = false  # Always false, because User's guide does not define it
-	varray_cell_center = nil
-	[result, varray_cell_center]
-      end
+	loopdimids = Array.new
+	loopdimnames = Array.new
+	loopdims.each{|d|
+	  case d
+	  when Integer
+	    if d < 0
+	      d += gp.rank
+	    end
+	    loopdimids.push( d )
+	    loopdimnames.push( gp.axis(d).name )
+	  when String
+	    loopdimids.push( gp.dim_index(d) )
+	    loopdimnames.push( d )
+	  else
+	    raise ArgumentError,"loopdims must consist of Integer and/or String"
+	  end
+	}
+
+	sh = Array.new
+	len = 1
+	loopdimids.each{|i| 
+	  sh.push(gp.shape[i])
+	  len *= gp.shape[i]
+	}
+
+	cs = [1]
+	(1...sh.length).each{|i| cs[i] = sh[i-1]*cs[i-1]}
+	idx_hash = Hash.new
+	for i in 0...len do
+	  loopdimnames.each_with_index{|d,j| 
+	    idx_hash[d] = ((i/cs[j])%sh[j])..((i/cs[j])%sh[j]) # rank preserved
+	  }
+	  subs = gphyses.collect{|g| g[idx_hash] }
+	  results = yield(*subs)
+	  if !results.is_a?(Array)
+	    raise "The return value of the block must be an Array of GPhys" 
+	  end
+	  if i == 0
+	    fl = files.shift
+	    results_whole = Array.new
+	    for j in 0...results.length
+	      rs = results[j]
+	      grid = rs.grid_copy
+	      loopdimnames.each{|nm| 
+		if !grid.axnames.include?( nm )
+		  raise "Dimension '#{nm}' has been eliminated. "+
+                        "You must keep all loop dimensions." 
+		end
+		grid.set_axis(nm,gphyses[j].axis(nm))
+	      }
+	      io_module.write_grid(fl, grid)
+	      fl = files.shift if files.length >= 1
+	      results_whole.push( 
+		io_module.def_var(fl, rs.name, rs.data.ntype, 
+				  grid.axnames, rs.data)
+	      )
+	    end
+	  end
+	  idx = grid.axnames.collect{|k| idx_hash[k] || true}
+	  for j in 0...results.length
+	    rs = results[j]
+	    rs.data.copy( results_whole[j][*idx] )
+	  end
+	end
+	return results_whole
 
-      def cell_center?(coord_var)
-	# whether the coordinate variale represent grid cell centers.
-	# coordvar (VArray)
-	# return value:
-	#    false if not
-        #    true if true and the corresponding cell bounds are not identified.
-        #    a VArray if true and the bounds are found (returns it)
-	result = false  # Always false, because User's guide does not define it
-	varray_cell_bounds = nil
-	[result, varray_cell_bounds]
       end
     end
 
@@ -169,19 +374,11 @@ module NumRu
 
       module_function
 
-      @@convention = NetCDF_Convention_Users_Guide
-
-      def set_convention(convention)
-	@@convention = convention
-      end
-      def convention
-	@@convention
-      end
-
       def open(files, varname)
 	files, ncvar0 = __interpret_files( files, varname )
 	data = __files2varray( files, varname )
-	axposnames = @@convention::coord_var_names(ncvar0)
+	convention = NetCDF_Conventions.find( ncvar0.file )
+	axposnames = convention::coord_var_names( ncvar0 )
 	rank = data.rank
 	bare_index = [ false ] * rank # will be true if coord var is not found
 
@@ -192,25 +389,27 @@ module NumRu
 	    axpos = __files2varray( files, axposnames[i], i )
 	  else
 	    bare_index[i]=true
-	    na = NArray.float(ncvar0.file.dim(dimnames[i]).length).indgen!
+	    na = NArray.float(ncvar0.shape_current[i]).indgen!
 	    axpos = VArray.new( na )
 	  end
-	  cell_center, varray_cell_bounds = @@convention::cell_center?( axpos )
-	  cell_bounds, varray_cell_center = @@convention::cell_bounds?( axpos )
+	  cell_center, cell_bounds_name = convention::cell_center?( axpos )
+	  cell_bounds, cell_center_name = convention::cell_bounds?( axpos )
 	  cell = cell_center || cell_bounds
 	  axis = Axis.new(cell,bare_index[i])
 	  if !cell
 	    axis.set_pos( axpos )
 	  else
 	    if cell_center
-	      if varray_cell_bounds
+	      if cell_bounds_name
+		varray_cell_bounds = __files2varray(files, cell_bounds_name, i)
 		axis.set_cell(axpos, varray_cell_bounds).set_pos_to_center
 	      else
 		p "cell bounds are guessed"
 		axis.set_cell_guess_bounds(axpos).set_pos_to_center
 	      end
 	    else  # then it is cell_bounds
-	      if varray_cell_center
+	      if cell_center_name
+		varray_cell_center = __files2varray(files, cell_center_name, i)
 		axis.set_cell(varray_cell_center, axpos).set_pos_to_bounds
 	      else
 		p "cell center is guessed"
@@ -219,7 +418,12 @@ module NumRu
 	    end
 	  end
 	  
-	  #p "yet-to-be-defined: method to define aux coord vars"
+	  aux = convention::aux_var_names( axpos )
+	  if aux
+	    aux.each{|k,varname| 
+	      axis.set_aux( k, __files2varray(files,varname,i) )
+	    }
+	  end
 	  
 	  axes[i] = axis
 	end
@@ -229,9 +433,9 @@ module NumRu
 	GPhys.new(grid,data)
       end
 
-      def write(file, gphys, name=nil)
-	(0...(gphys.rank)).each{|i|
-	  ax = gphys.axis(i)
+      def write_grid(file, grid_or_gphys)
+	(0...(grid_or_gphys.rank)).each{|i|
+	  ax = grid_or_gphys.axis(i)
 	  dimname = ax.pos.name
 	  length = ax.pos.length
 	  isfx = 0
@@ -259,10 +463,25 @@ module NumRu
 	    end
 	  }
 	}
+	nil
+      end
+
+      def def_var(file, name, ntype, dimensions, vary=nil)
+	VArrayNetCDF.def_var(file, name, ntype, dimensions, vary)
+      end
+
+      def write(file, gphys, name=nil)
+        write_grid(file, gphys)
+	NetCDF_Conventions.add_history(file, "#{self}.write "+gphys.name)
 	VArrayNetCDF.write(file, gphys.data, name, gphys.axnames)
 	nil
       end
 
+      def each_along_dims_write(gphyses, files, *loopdims, &block)
+	IO_Common::each_along_dims_write(gphyses, files, loopdims, NetCDF_IO,
+					 &block)
+      end
+
       ############################################################
 
       def __files2varray( files, varname, dim=nil )
@@ -271,7 +490,6 @@ module NumRu
 	  file = files
 	  var = file.var( varname )
 	  raise "variable '#{varname}' not found in #{file}" if !var
-	  # @@convention::interpret_miss_handling_and_scaling(var)
 	  VArrayNetCDF.new( var )
 	elsif files.is_a?(NArray)
 	  # Suppose that files is a NArray of NetCDF. Returns a VArrayCompsite.
@@ -282,7 +500,6 @@ module NumRu
 	  for i in 0...files.length
 	    var = files[i].var( varname )
 	    raise "variable '#{varname}' not found in #{files[i].path}" if !var
-	    # @@convention::interpret_miss_handling_and_scaling(var)
 	    varys[i] = VArrayNetCDF.new( var )
 	  end
 	  if files.length != 1
@@ -323,7 +540,8 @@ module NumRu
 	if files.rank > ncvar0.rank
 	  raise ArgumentError, "rank of files > rank of data"
 	end
-	axposnames = @@convention::coord_var_names(ncvar0)
+	convention = NetCDF_Conventions.find( ncvar0.file )
+	axposnames = convention::coord_var_names(ncvar0)
 
 	#< find correspoding dimensions >
 
@@ -381,7 +599,7 @@ module NumRu
 	    files.each{|f|
 	      raise TypeError, "non-String obj included" if !f.is_a?(String)
 	    }
-	    for i in 0...file.length
+	    for i in 0...files.length
 	      files[i] = NetCDF.open(files[i])
 	    end
 	  end
@@ -460,6 +678,7 @@ if $0 == __FILE__
 
    temp_edy = ( temp - temp_xmean )
    p '###',temp_edy.name,temp_edy.val[0,true,true]
+   p 'deleted attributes:', temp.data.att_names - temp_edy.data.att_names
    p '@@@',temp
    p '///',temp.copy
    p '+++',temp2
@@ -489,5 +708,25 @@ if $0 == __FILE__
    f.close
    files = /tmp(\d)(\d).nc/
    p gpcompo = GPhys::NetCDF_IO.open( files, 'T' )
-   p gpcompo.coord(0).val, gpcompo[false,0].val
+   p gpcompo.coord(0).val
+   p gpcompo[false,0].val
+
+
+  p '** test each_along_dims* **'
+
+  f=NetCDF.create('tmpE1.nc')
+  GPhys::NetCDF_IO.each_along_dims_write( temp, f, 1, 2 ){|sub|
+    [sub.mean(0)]
+  }
+  f.close
+  f=NetCDF.create('tmpE2.nc')
+  GPhys::NetCDF_IO.each_along_dims_write([temp,temp_edy], f, "level"){|s1,s2|
+    [s1.mean(0),s2.mean(1).rename('T_edy')]
+  }
+  f.close
+  f=NetCDF.create('tmpE0.nc')
+  GPhys::NetCDF_IO.write( f, temp.mean(0) )
+  f.close
+
+  print `ncdump tmpE0.nc > tmpE0; ncdump tmpE1.nc > tmpE1 ; diff -u tmpE[01]`
 end
