@@ -44,12 +44,22 @@ module NumRu
 	@lost_axes.dup
       end
 
-      def axis(i)
-	 @axes[i]
+#      def axis(i)
+#	 @axes[i]
+#      end
+
+      def axis(dim_or_dimname)
+	 ax_dim(dim_or_dimname)[0]
       end
 
-      def set_axis(i,ax)
-	 @axes[i] = ax
+      alias get_axis axis
+
+      def dim_index(dimname)
+	 ax_dim(dimname)[1]
+      end
+
+      def set_axis(dim_or_dimname,ax)
+	 @axes[ ax_dim(dim_or_dimname)[1] ] = ax
       end
 
       def set_lost_axes( lost )
@@ -97,8 +107,10 @@ module NumRu
 	  msg = '('+deleted_by+') '
 	end
 	lost = at.collect{|pos|
+	  mn = sprintf("%g", ( UNumeric===(a=@axes[pos].pos.min) ? a.val : a) )
+	  mx = sprintf("%g", ( UNumeric===(a=@axes[pos].pos.max) ? a.val : a) )
 	  msg + 
-	  "#{@axes[pos].name}:#{sprintf("%g",@axes[pos].pos.min)}..#{sprintf("%g",@axes[pos].pos.max)}"
+	  "#{@axes[pos].name}:#{mn}..#{mx}"
 	}
 
 	newgrid.add_lost_axes( lost )
@@ -203,14 +215,6 @@ module NumRu
       end
       private :_cut_
 
-      def get_axis(dim_or_dimname)
-	 ax_dim(dim_or_dimname)[0]
-      end
-
-      def dim_index(dim_or_dimname)
-	 ax_dim(dim_or_dimname)[1]
-      end
-
       def exclude(dim_or_dimname)
 	 dim = dim_index(dim_or_dimname)
 	 axes = @axes.dup
@@ -230,19 +234,20 @@ module NumRu
 	 Grid.new( *axes ).add_lost_axes( lost )
       end
 
-      def change_axis!(dim, axis)
+      def change_axis!(dim_or_dimname, axis)
 	 if axis.is_a?(Axis)
-	    @axes[dim] = axis
+	    @axes[ ax_dim(dim_or_dimname)[1] ] = axis
 	 else
 	    @lost_axes.push(axis) if axis.is_a?(String)
-	    @axes.delete_at(dim)
+	    @axes.delete_at( ax_dim(dim_or_dimname)[1] )
 	 end
 	 @rank = @axes.length
 	 __check_and_set_axnames
 	 self
       end
 
-      def insert_axis(dim, axis)
+      def insert_axis(dim_or_dimname, axis)
+	 dim = ax_dim(dim_or_dimname)[1]
 	 axes = @axes.dup
 	 if axis.is_a?(Axis)
 	    axes[dim+1,0] = axis    # axes.insert(dim, axis) if ruby 1.7
@@ -252,7 +257,8 @@ module NumRu
 	 Grid.new( *axes )
       end
 
-      def insert_axis!(dim, axis)
+      def insert_axis!(dim_or_dimname, axis)
+	 dim = ax_dim(dim_or_dimname)[1]
 	 if axis == nil
 	    # do nothing
 	 else
@@ -271,7 +277,7 @@ module NumRu
 	 def #{method}(vary, dim_or_dimname, *extra_args)
 	    ax, dim = self.ax_dim(dim_or_dimname)
 	    va, new_ax = ax.#{method}(vary, dim, *extra_args)
-            if va.is_a?(Numeric)
+            if va.is_a?(Numeric) || va.is_a?(UNumeric)
               va
 	    else
               [va, self.change_axis(dim, new_ax)]
